#!/usr/bin/python3

# import libraries
from sklearn.preprocessing import StandardScaler
from sklearn.neural_network import MLPRegressor
from sklearn.metrics import classification_report, confusion_matrix
from sklearn.model_selection import train_test_split
import os,sys, numpy as np, pickle


#######  SETTINGS / ADJUST  ##########################################
infile = "train_data.txt"
hidden = (55,85,55)  # basic: 20,35,20 # large (55,85,55) #small (5,8,5)
iters = 1000         # basic: 500 #large 2000 # small 200
#######################################################################


print( "hidden: ", hidden, "iters", iters, "infile: ", infile)

#  read data from file and fill data matrix X and reference values y
print( "read data" )
x = []
y = []
with open( infile ) as f:
        for l in f:
                c = l.split()
                x.append( [ float(c[0]) ] )
                y.append( float(c[1]) )
                
print( "data points: ", len(x), len(y))

##  SPLIT DATASET INTO TRAINING AND TEST DATA 
x_train, x_test, y_train, y_test = train_test_split( x, y, test_size = 0.2)
print( "split:", len(y_train), len(y_test))


## SCALE DATA 
#print( "scale data...")
#scaler = StandardScaler()
#scaler.fit(x_train)
#x_train = scaler.transform(x_train)
#x_test = scaler.transform(x_test)


####  MAIN PART  ####
print( "set up ANN")
ann = MLPRegressor( hidden_layer_sizes=hidden, max_iter=iters)

print( "training...")
sys.stdout.flush()
ann.fit( x_train, y_train)


print( "evaluate...")
# use trained ANN to calculate values for test data
predictions = ann.predict(x_test)

# write predicted values into file
error = 0
with open( "test_predictions.txt", 'w') as w:
        for x,y,z in zip( x_test,predictions,y_test):
                error += np.square( y-z) 
                print( x[0], y, z, file=w)
print( "error:", np.sqrt(error/len(x_test)))

# output of ann to use as predictor
pickle.dump( ann, open('ann.pkl','wb'))




