#!/usr/bin/python3

# copyright rene staritzbichler, 31.8.2020


import sys, os


# sorted by kyte doolitle hydrophicity scale:
def sort_by_hydrophobicity():
    aa2id = {}
    aa2id["Arg".upper()] = 1  
    aa2id["Lys".upper()] = 2  
    aa2id["Asn".upper()] = 3  
    aa2id["Asp".upper()] = 4  
    aa2id["Gln".upper()] = 5  
    aa2id["Glu".upper()] = 6  
    aa2id["His".upper()] = 7  
    aa2id["Pro".upper()] = 8  
    aa2id["Tyr".upper()] = 9  
    aa2id["Trp".upper()] = 10  
    aa2id["Ser".upper()] = 11  
    aa2id["Thr".upper()] = 12  
    aa2id["Gly".upper()] = 13  
    aa2id["Ala".upper()] = 14  
    aa2id["Met".upper()] = 15  
    aa2id["Cys".upper()] = 16  
    aa2id["Phe".upper()] = 17  
    aa2id["Leu".upper()] = 18  
    aa2id["Val".upper()] = 19  
    aa2id["Ile".upper()] = 20
#    aa2id["MSE"] = 21 # sort of crap!
    return aa2id
    
def grouping():

    # 0 : PRO
    # 1 : GLY
    # 2 : Cys, Sec
    # 3 : Arg, His, Lys       pos
    # 4 : Asp, Glu            neg
    # 5 : Ser, Thr, Asn, Gln  polar
    # 6 : Phe, Tyr, Trp       aromatic, hydrophobic
    # 7 : Ala, Val, Ile, Leu, Met       hydrophobic

    aa2id = {}
    aa2id["Arg".upper()] = [1 ,3]    
    aa2id["Lys".upper()] = [2 ,3] 
    aa2id["Asn".upper()] = [3 ,5] 
    aa2id["Asp".upper()] = [4 ,4] 
    aa2id["Gln".upper()] = [5 ,5] 
    aa2id["Glu".upper()] = [6 ,4] 
    aa2id["His".upper()] = [7 ,3] 
    aa2id["Pro".upper()] = [8 ,0] 
    aa2id["Tyr".upper()] = [9 ,6] 
    aa2id["Trp".upper()] = [10,6]  
    aa2id["Ser".upper()] = [11,5]  
    aa2id["Thr".upper()] = [12,5]  
    aa2id["Gly".upper()] = [13,1]  
    aa2id["Ala".upper()] = [14,7]  
    aa2id["Met".upper()] = [15,7]  
    aa2id["Cys".upper()] = [16,2]  
    aa2id["Phe".upper()] = [17,6]  
    aa2id["Leu".upper()] = [18,7]  
    aa2id["Val".upper()] = [19,7]  
    aa2id["Ile".upper()] = [20,7]
#    aa2id["MSE"] = 21 # sort of crap!
    return aa2id
    
def profile():

    # n values per residue:
    # id, polar, pos/neg, aromatic, rest/gly/cys/his/pro , (kyte-doolittle)
    
    aa2id = {}
    aa2id["Arg".upper()] = [1 , 2 , 1 , 0 , 0  ]    
    aa2id["Lys".upper()] = [2 , 2 , 1 , 0 , 0  ] 
    aa2id["Asn".upper()] = [3 , 1 , 0 , 0 , 0  ] 
    aa2id["Asp".upper()] = [4 , 2, -1 , 0 , 0  ] 
    aa2id["Gln".upper()] = [5 , 1 , 0 , 0 , 0  ] 
    aa2id["Glu".upper()] = [6 , 2 ,-1 , 0 , 0  ] 
    aa2id["His".upper()] = [7 , 2 , 1 , 1 , 3  ] 
    aa2id["Pro".upper()] = [8 , 0 , 0 , 1 , 4  ] 
    aa2id["Tyr".upper()] = [9 , 0 , 0 , 1 , 0  ] 
    aa2id["Trp".upper()] = [10, 0 , 0 , 1 , 0  ]  
    aa2id["Ser".upper()] = [11, 1 , 0 , 0 , 0  ]  
    aa2id["Thr".upper()] = [12, 1 , 0 , 0 , 0  ]  
    aa2id["Gly".upper()] = [13, 0 , 0 , 0 , 1  ]  
    aa2id["Ala".upper()] = [14, 0 , 0 , 0 , 0  ]  
    aa2id["Met".upper()] = [15, 0 , 0 , 0 , 0  ]  
    aa2id["Cys".upper()] = [16, 0 , 0 , 0 , 2  ]  
    aa2id["Phe".upper()] = [17, 0 , 0 , 1 , 0  ]  
    aa2id["Leu".upper()] = [18, 0 , 0 , 0 , 0  ]  
    aa2id["Val".upper()] = [19, 0 , 0 , 0 , 0  ]  
    aa2id["Ile".upper()] = [20, 0 , 0 , 0 , 0  ]
#    aa2id["MSE"] = 21 # sort of crap!
    return aa2id
    
    
def unsorted_dict():
    aa2id = {}
    aa2id["Ala".upper()] = 1   
    aa2id["Arg".upper()] = 2   
    aa2id["Asn".upper()] = 3   
    aa2id["Asp".upper()] = 4   
    aa2id["Cys".upper()] = 5   
    aa2id["Gln".upper()] = 6   
    aa2id["Glu".upper()] = 7   
    aa2id["Gly".upper()] = 8   
    aa2id["His".upper()] = 9   
    aa2id["Ile".upper()] = 10  
    aa2id["Leu".upper()] = 11  
    aa2id["Lys".upper()] = 12  
    aa2id["Met".upper()] = 13  
    aa2id["Phe".upper()] = 14  
    aa2id["Pro".upper()] = 15  
    aa2id["Ser".upper()] = 16  
    aa2id["Thr".upper()] = 17  
    aa2id["Trp".upper()] = 18  
    aa2id["Tyr".upper()] = 19  
    aa2id["Val".upper()] = 20  
#    aa2id["MSE"] = 21 # sort of crap!
    return aa2id

def single_letter():
    aa321 = {}
    aa321[ "Gly".upper() ] = "G"                                     
    aa321[ "Ala".upper() ] = "A"                                     
    aa321[ "Leu".upper() ] = "L"                                     
    aa321[ "Met".upper() ] = "M"                                     
    aa321[ "Phe".upper() ] = "F"                                     
    aa321[ "Trp".upper() ] = "W"                                     
    aa321[ "Lys".upper() ] = "K"                                     
    aa321[ "Gln".upper() ] = "Q"                                     
    aa321[ "Glu".upper() ] = "E"                                     
    aa321[ "Ser".upper() ] = "S"                     
    aa321[ "Pro".upper() ] = "P" 
    aa321[ "Val".upper() ] = "V" 
    aa321[ "Ile".upper() ] = "I"       
    aa321[ "Cys".upper() ] = "C"    
    aa321[ "Tyr".upper() ] = "Y"    
    aa321[ "His".upper() ] = "H"    
    aa321[ "Arg".upper() ] = "R"    
    aa321[ "Asn".upper() ] = "N"                   
    aa321[ "Asp".upper() ] = "D"     
    aa321[ "Thr".upper() ] = "T"   
    return aa321


def convert( filename, chain, outname, mode="hb"):
    sequence = []
    helices = []
    id2name = {}
    previd = -999
    if mode == "sort_by_hydrophobicity":
        aa2id = sort_by_hydrophobicity()
    elif mode == "single":
        aa2id = single_letter()
    elif mode == "grouping":
        aa2id = grouping()
    elif mode == "profile":
        aa2id = profile()
        
    with open( filename) as f:
        for l in f:
            if "SEQRES" == l[:6] and l[11] == chain:
                sequence.extend( l[19:].split() )
            if "HELIX" == l[:5] and l[19] == chain and l[31] == chain:
                helices.append( [ int(l[21:25]) , l[15:18], int(l[33:37]), l[27:30] ] )

            if "ATOM" == l[:4] and l[21] == chain:
                resid = int( l[22:26] )
                resname = l[17:20].upper()
                if resid != previd:
                    id2name[resid] = resname
                    previd = resid
                    if resname not in aa2id:
                        print( "WARNING: unknown residue name: ", resname)
                        exit(1)
    #print( "length: seq:", len(sequence), len(helices), len(id2name))
    # check consistency:
    for h in helices:
        if id2name[h[0]] != h[1]:
            print( "WARNING: mismatch1! expected:", h[0],h[1], "got:", id2name[h[0]] )
            exit(2)
        if id2name[h[2]] != h[3]:
            print( "WARNING: mismatch2! expected:", h[2],h[3], "got:", id2name[h[2]] )
            exit(3)
        if __name__ == "__main__":
            print( "HELIX: ", h[0],h[1], h[2],h[3] )
    with open( outname, 'w') as w:
        for di, name in id2name.items():
            in_helix = "0"
            for h in helices:
                if di >= h[0] and di <= h[2]:
                    in_helix = "1"
                    break
                
            if __name__ == "__main__":
                print( di, name,  in_helix)
            if mode != "grouping" and mode != "profile":
                w.write( str(aa2id[name]) + ' ' + in_helix + '\n')
            else:
                xy = aa2id[name]
                for z in xy:
                    w.write( str(z) + ' ' )
                w.write( in_helix + '\n')
    
    


    
if __name__ == "__main__":
    convert( sys.argv[1], sys.argv[2], sys.argv[3] )
    
