#!/usr/bin/python3

from sklearn.preprocessing import StandardScaler
from sklearn.neural_network import MLPClassifier
from sklearn.metrics import classification_report, confusion_matrix
from sklearn.model_selection import train_test_split
import os,sys


#######  SETTINGS :: ADJUST HERE ############
indir = "db/div3_2"   ###  SELECT INPUT DATA SET
hidden = (100)        ###  SELECT ARCHITECTURE
iters = 100           ###  SELECT LENGTH OF OPTIMIZATION
##############################################



print( "hidden: ", hidden, "iters", iters, "indir: ", indir)

# read data matrix X and reference data y (expected results):
x = []
y = []
for name in os.listdir(indir):
    if '.txt' not in name: continue
    with open( os.path.join( indir, name ) ) as f:
        for l in f:
            c = l.split()
            x.append( [float(x) for x in c[:-1]] )
            y.append( float(c[-1]) )

print( "data points: ", len(x), len(y))

# split X and y into train and test data 
x_train, x_test, y_train, y_test = train_test_split( x, y, test_size = 0.2)

print( "split:", len(y_train), len(y_test))
print( "balance pos/neg in training data set:", y_train.count(1), y_train.count(0) )
print( "balance pos/neg in test data set:", y_test.count(1), y_test.count(0) )

# scaling and centering often helps learning machines in learning patterns
print( "scale data...")
scaler = StandardScaler()
scaler.fit(x_train)
x_train = scaler.transform(x_train)
x_test = scaler.transform(x_test)

# set up ANN
print( "set up ANN")
ann = MLPClassifier( hidden_layer_sizes=hidden, max_iter=iters)


print( "training...")
sys.stdout.flush()

# train ANN
ann.fit( x_train, y_train)


print( "evaluate...")
# calculate predictions of test data
predictions = ann.predict(x_test)

# print confusion matrix
print("\n",confusion_matrix(y_test,predictions))
print(classification_report(y_test,predictions))
